# TPU k-Means

## Prerequisites:

- Google Cloud Project exists*
- Google Cloud Bucket exists*
- Google Cloud SDK Shell is installed on local machine
- PuTTY is installed on local machine
- Python3 is installed
- Access to the git repository https://git01lab.cs.univie.ac.at/wolffa95/tpu-k-means


*In the following, replace <PROJECT_ID> or <PROJECT_NAME> and <BUCKET_NAME> by your project and bucket.

[comment]: <> (│   └── file23.ext)


## Google Project Setup

### Setup the Google Project on Local Machine

Open the Google Cloud SDK Shell

Update components `gloud compontents update`

#### Init gcloud

`gloud init`

Follow on-screen instructions

### Create a new project

Instructions can be found [here](https://cloud.google.com/tpu/docs/setup-gcp-account)

### Set project
`gloud config set project <PROJECT_NAME>`

### Create a cloud storage bucket
Detailed instructions can be found [here](https://cloud.google.com/tpu/docs/storage-buckets)

The default bucket name in the project is *k-means-bucket* and is recommended to use.

The bucket must have the following structure:

```
.
├── evaluation
├── external
├── processed
└── raw
```

### Create service account for the Cloud TPU project
`gcloud beta services identity create --service tpu.googleapis.com --project <PROJECT_ID>`

Result: service identity created service-<PROJECT_ID>@cloud-tpu.iam.gserviceaccount.com

Then follow the instructions on [Google](https://cloud.google.com/docs/authentication).

Once the service account is set up, the json file must be placed in the **tpu-k-means** project folder, and added in _authenticate.py_.

### Show project
`gcloud projects describe <PROJECT_ID>`

## Setup of k-Means Project

### Clone the project and install dependencies

`git clone https://git01lab.cs.univie.ac.at/wolffa95/tpu-k-means.git`


`pip install -r requirements.txt`


### Adapt variables in script (main.py)

- INIT_CENTROIDS_FIXED = True / False
- FILENAME
- LABELS_EXIST = True / False
- EPOCHS: number of iterations the first version runs (fixed number of iterations)
- MAX_EPOCHS: number of iterations the second version runs (not fixed)
- TOLERANCE: of convergence
- BATCH_SIZE_PER_REPLICA: choose appropriate for the data set

### Additional starting points for the implemenation

- _main.py_: Contains the optimal version
- _main_combine.py_: Combines v1 -> v2 or v2 -> v1, depending on which version is set as first
- _main_combine_loop.py_: Combines v1 -> v2 or v2 -> v1, depending on which version is set as first, and loops over all synthetic or real datasets. It must be specified whether the synthetic or real data sets should be used. It is important to set the correct directory of the files.
- _main_combine_loop2.py_: main_combine_loop.py adapted for Experiment 4 part 2 and 3
- _main_repeat.py_: Executes version 1, resets the centroids to the initial centroids, and repeats the execution with version 2
- _main_repeat_loop.py_: Executes version 1, resets the centroids to the initial centroids, and repeats the execution with version 2, and loops over all synthetic or real data sets. It furthermore, loops over the iterations array and switches between max iterations fixed or not fixed.
- _main_repeat_loop2.py_: main_repeat_loop.py adapted for Experiment 4 part 2 and 3
- _main_repeat_loop3.py_: main_repeat_loop.py adapted for Experiment 5
- _main_repeat_batch_loop.py_: main_repeat_loop.py adapted for Experiment 1

## Run on TPU
Open the Google Cloud SDK Shell

Update components `gloud compontents update`

<details open>
<summary style="color:darkgray;font-size:20px;"><b>Run on TPU (if it does not exist)</b></summary>

### Create TPU VM
**A: TPU v2**
`gcloud alpha compute tpus tpu-vm create tpu-kmeans --zone=us-central1-b --accelerator-type=v2-8 --version=tpu-vm-tf-2.8.0`

*if there is no more capacity in the zone:* `gcloud alpha compute tpus tpu-vm create tpu-kmeans2 --zone=us-central1-c --accelerator-type=v2-8 --version=tpu-vm-tf-2.8.0`

**B: TPU v3**
`gcloud alpha compute tpus tpu-vm create tpu-kmeans3 --zone=us-central1-b --accelerator-type=v3-8 --version=tpu-vm-tf-2.8.0`

### Connect to Cloud TPU VM
`gcloud alpha compute tpus tpu-vm ssh tpu-kmeans --zone=us-central1-b`

*or:* `gcloud alpha compute tpus tpu-vm ssh tpu-kmeans2 --zone=us-central1-c`

*or:* `gcloud alpha compute tpus tpu-vm ssh tpu-kmeans3 --zone=us-central1-b`

PuTTY opens

**------------------------------ Continue on VM ------------------------------**
### Create an environment variable for the TPU
`export TPU_NAME=local`

### Create environment variables for Google Cloud Bucket
`export STORAGE_BUCKET=gs://<BUCKET_NAME>`

For synthetic data sets: `export DATA_DIR=${STORAGE_BUCKET}/raw` or

For real data sets: `export DATA_DIR=${STORAGE_BUCKET}/external`

### Get Python Scripts from git
`git clone https://git01lab.cs.univie.ac.at/wolffa95/tpu-k-means.git`

`cd tpu-k-means`

### Save git credentials
`git config credential.helper store`

`git config --global user.email "<you@example.com>`

`git config --global user.name "<Your Name>"`

### Install requirements
`pip3 install -r requirements.txt`

### Create temporary directory for results
`mkdir results_tmp` (only when running main.py)

### Execute Python Script
`python3 main.py --data_dir=${DATA_DIR} --tpu_name=${TPU_NAME}`

### Delete temporary directory
`rm -r results_tmp` (only when running main.py)

### Push log files to git

#### If running for the first time today
`cd logs`

`git add <yyyy-mm-dd>.log`

`git commit -a -m "new log file"`

`git push`

#### else
`git commit -a -m "updated log file"`

`git push`

### Shutdown TPU
`exit`

**-------------------- Continue on Google Cloud SDK Shell --------------------**

### Stop or delete TPU
**A: Stop**
`gcloud alpha compute tpus tpu-vm stop tpu-kmeans --zone=us-central1-b`

*or:* `gcloud alpha compute tpus tpu-vm stop tpu-kmeans2 --zone=us-central1-c`

*or:* `gcloud alpha compute tpus tpu-vm stop tpu-kmeans3 --zone=us-central1-b`

**B: Delete**
`gcloud alpha compute tpus tpu-vm delete tpu-kmeans --zone=us-central1-b`

*or:* `gcloud alpha compute tpus tpu-vm delete tpu-kmeans2 --zone=us-central1-c`

*or:* `gcloud alpha compute tpus tpu-vm delete tpu-kmeans3 --zone=us-central1-b`

### Verify
`gcloud alpha compute tpus tpu-vm list --zone=us-central1-b`

*or:* `gcloud alpha compute tpus tpu-vm list --zone=us-central1-c`


### View results on cloud bucket
https://console.cloud.google.com/storage/browser/<BUCKET_NAME>;tab=objects?forceOnBucketsSortingFiltering=false&project=<PROJECT_ID>&prefix=&forceOnObjectsSortingFiltering=false

subdirectories: /processed and /evaluation

</details>

<details open>
<summary style="color:darkgray;font-size:20px;"><b>Run on TPU (if it already exists)</b></summary>

### Start TPU VM
**A: TPU v2**
`gcloud alpha compute tpus tpu-vm start tpu-kmeans --zone=us-central1-b`

*if there is no more capacity in the zone:* `gcloud alpha compute tpus tpu-vm start tpu-kmeans2 --zone=us-central1-c`

**B: TPU v3**
`gcloud alpha compute tpus tpu-vm start tpu-kmeans3 --zone=us-central1-b`

### Connect to Cloud TPU VM
`gcloud alpha compute tpus tpu-vm ssh tpu-kmeans --zone=us-central1-b`

*or:* `gcloud alpha compute tpus tpu-vm ssh tpu-kmeans2 --zone=us-central1-c`

*or:* `gcloud alpha compute tpus tpu-vm ssh tpu-kmeans3 --zone=us-central1-b`

PuTTY opens

**------------------------------ Continue on VM ------------------------------**

### Create an environment variable for the TPU
`export TPU_NAME=local`

### Create environment variables for Google Cloud Bucket
`export STORAGE_BUCKET=gs://<BUCKET_NAME>`

For synthetic data sets: `export DATA_DIR=${STORAGE_BUCKET}/raw` or

For real data sets: `export DATA_DIR=${STORAGE_BUCKET}/external`

### Update Python Script from git
`cd tpu-k-means`

`git pull`

### Create temporary directory for results
`mkdir results_tmp` (only when running main.py)

### Execute Python Script
`python3 main.py --data_dir=${DATA_DIR} --tpu_name=${TPU_NAME}`

### Delete temporary directory
`rm -r results_tmp` (only when running main.py)

### Push log files to git

**If running for the first time today**

`cd logs`

`git add <yyyy-mm-dd>_log.log`

`git commit -a -m "new log file"`

`git push`

**Else**

`git commit -a -m "updated log file"`

`git push`

### Shutdown TPU
`exit`

**-------------------- Continue on Google Cloud SDK Shell --------------------**
### Stop or delete TPU
**A: Stop**
`gcloud alpha compute tpus tpu-vm stop tpu-kmeans --zone=us-central1-b`

*or:* `gcloud alpha compute tpus tpu-vm stop tpu-kmeans2 --zone=us-central1-c`

*or:* `gcloud alpha compute tpus tpu-vm stop tpu-kmeans3 --zone=us-central1-b`

**B: Delete**
`gcloud alpha compute tpus tpu-vm delete tpu-kmeans --zone=us-central1-b`

*or:* `gcloud alpha compute tpus tpu-vm delete tpu-kmeans2 --zone=us-central1-c`

*or:* `gcloud alpha compute tpus tpu-vm delete tpu-kmeans3 --zone=us-central1-b`

### Verify
`gcloud alpha compute tpus tpu-vm list --zone=us-central1-b`

*or:* `gcloud alpha compute tpus tpu-vm list --zone=us-central1-c`


### View results on cloud bucket
https://console.cloud.google.com/storage/browser/<BUCKET_NAME>;tab=objects?forceOnBucketsSortingFiltering=false&project=<PROJECT_ID>&prefix=&forceOnObjectsSortingFiltering=false

subdirecotries: /processed and /evaluation

</details>

## Run on CPU

### Create temporary directory for results
`mkdir results_tmp`

### Execute Python Script
`python main.py --data_dir=<location of data>` (default: kmeans-bucket/raw)

### Delete temporary directory
`rm results_tmp`

### Upload log files to git

**If running for the first time today**

`cd logs`

`git add <yyyy-mm-dd>_log.log`

`git commit -a -m "new log file"`

`git push`

**Else**
`git commit -a -m "updated log file"`

`git push`

### View results on cloud bucket
https://console.cloud.google.com/storage/browser/<BUCKET_NAME>;tab=objects?forceOnBucketsSortingFiltering=false&project=<PROJECT_ID>&prefix=&forceOnObjectsSortingFiltering=false

subdirectories: /processed and /evaluation


## Run on GPU

### Create temporary directory for results
`mkdir results_tmp`

### Set number of GPUs available
Adapt the main python script to the number of GPUs available

### Execute Python Script
`python3 main.py --data_dir=<location of data>` (default: kmeans-bucket/raw)

### Delete temporary directory
`rm results_tmp`

### Upload log files to git

**If running for the first time today**

`cd logs`

`git add <yyyy-mm-dd>_log.log`

`git commit -a -m "new log file"`

`git push`

**Else**
`git commit -a -m "updated log file"`

`git push`

### View results on cloud bucket
https://console.cloud.google.com/storage/browser/<BUCKET_NAME>;tab=objects?forceOnBucketsSortingFiltering=false&project=<PROJECT_ID>&prefix=&forceOnObjectsSortingFiltering=false

subdirectories: /processed and /evaluation
