import argparse
import logging


def user_select_dataset(options):
    """user selects data set on which k-Means should be applied"""
    # options: ["data_8_460800_8", "data_32_1024000_64", "data_128_5017600_128"]
    print("Please choose an input data set:")
    for idx, element in enumerate(options):
        _, n_clusters, n_objects, n_features = element.split("_", 3)
        print("{}) data set with {} samples, {} features, {} clusters".format(idx + 1, n_objects, n_features,
                                                                              n_clusters))
    i = input("Enter number: ")
    try:
        if 0 < int(i) <= len(options):
            return int(i)-1
        else:
            logging.warning("Number out of range. Default dataset selected.")
            print("Number out of range. Default dataset selected:")
            return 0
    except (ValueError, Exception):
        logging.error('Failed.', exc_info=True)
        print("An exception occurred. Default dataset selected:")
    return 0


def get_terminal_args():
    """data dir and tpu name arguments from terminal"""
    parser = argparse.ArgumentParser(description='Process data file paths')  # or without description ....
    parser.add_argument('--data_dir', type=str, default='gs://k-means-bucket/raw',
                        help='Directory for storing input data')
    parser.add_argument('--tpu_name', type=str, default=None,
                        help='Name of the TPU if running on a TPU', nargs='?')

    args, unparsed = parser.parse_known_args()
    data_dir = args.data_dir
    tpu_name = args.tpu_name
    print("data dir: {0}".format(data_dir))
    logging.info("data dir: {0}".format(data_dir))
    print("tpu name: {0}".format(tpu_name))
    logging.info("tpu name: {0}".format(tpu_name))

    return data_dir, tpu_name
